#!/usr/bin/env python
'''Main backend server object'''

__licence__ = "GPLv2"
__copyright__ = "2007, Lauri Taimila"
__author__ = "Lauri Taimila <lauri@taimila.com>"
__version__ = "0.1"

import os
import sys
import shutil

import gobject

# Configuration
from utils.configuration import Configuration

# Entertainer backend core
from backend.core.message import Message
from backend.core.message_bus import MessageBus
from backend.core.message_scheduler import MessageScheduler
from backend.core.connection_server import ConnectionServer
from backend.core.message_type_priority import MessageType, MessagePriority

# Entertainer backend components
from backend.components.feeds.feed_manager import FeedManager
from backend.components.tvguide.guide_updater import GuideUpdater
from backend.components.weather.weather_manager import WeatherManager
from backend.components.mediacache.media_cache_manager import MediaCacheManager

# Logging system
from utils.logger import Logger

class BackendServer:
    """
    Entertainer backend server.

    This is a backend of Entertainer media center. Backend is responsible of
    many vital things like updateing media library cache, recording scheduled
    TV-shows, fetching RSS-feeds etc.
    """

    def __init__(self):
        """
        Initialize backend

        This method creates and initializes all backend components.
        """
        gobject.threads_init()

        self.configuration = Configuration()
        self.logger = Logger().getLogger('backend.BackendServer')
        self.message_bus = MessageBus()

        # Notify object - used to display desktop notifications
        self.notifier = None
        # Connection server - Thread that listens incoming socket connections
        self.connection_server = None

        # Config files
        self.content_config = None
        self.preferences = None

        self.scheduler = None
        self.feed_manager = None
        self.guide_updater = None
        self.media_manager = None

        self.config_path = os.path.expanduser(self.configuration.get_cfg_path())

        # The order of the initialize method calls is significant! Don't change
        # the order unless you know what your doing!
        self.initialize_configuration()
        self.initialize_feed_manager()
        self.initialize_weather_manager()
        self.initialize_media_cache_manager()
        self.initialize_guide_updater()
        self.initialize_connection_server()
        self.initialize_scheduler()

    def initialize_configuration(self):
        """Initialize configuration"""
        try:
            cfg_dict = { MessageType.CONTENT_CONF_UPDATED :
                             MessagePriority.VERY_HIGH,
                         MessageType.PREFERENCES_CONF_UPDATED :
                             MessagePriority.VERY_HIGH }
            self.message_bus.registerMessageHandler(self.configuration,
                                                    cfg_dict)
            self.logger.debug("Configuration intialized successfully")
        except:
            print "Couldn't read configuration file! Execution aborted..."
            sys.exit(1)

    def initialize_connection_server(self):
        """Initialize connection server."""
        try:
            self._port = self.configuration.get_port()
            self.connection_server = ConnectionServer(self._port,
                                                      self.message_bus)
            # Start listening incoming connections
            self.connection_server.start()
        except:
            self.logger.error("ConnectionServer initialization failed!")
            message = 'Error occured. Please see '
            message += self.config_path + 'entertainer.log'
            message += ' for more information. This is probably not a fatal'
            message += ' error. Just wait one minute and try again.'
            print message
            print "Execution aborted!"
            sys.exit(1)

    def initialize_scheduler(self):
        """Initialize message scheduler."""
        self.scheduler = MessageScheduler(self.message_bus)
        self.scheduler.addMessage(Message(MessageType.UPDATE_FEEDS),
            60 * self.configuration.get_feed_fetch_interval())
        self.logger.debug("Message scheduler intialized successfully")

    def initialize_feed_manager(self):
        """Initialize feed manager."""
        self.feed_manager = FeedManager(self.message_bus, self.configuration)
        feed_mgr_dict = { MessageType.UPDATE_FEEDS : MessagePriority.HIGH }
        self.message_bus.registerMessageHandler(self.feed_manager,
                                                feed_mgr_dict)
        self.logger.debug("Feed Manager intialized successfully")

    def initialize_weather_manager(self):
        """Initialize feed manager."""
        self.weather_manager = WeatherManager(self.message_bus,
                                              self.configuration)
        weather_mgr_dict = { MessageType.UPDATE_WEATHER : MessagePriority.HIGH }
        self.message_bus.registerMessageHandler(self.weather_manager,
                                                weather_mgr_dict)
        self.logger.debug("Weather Manager intialized successfully")

    def initialize_guide_updater(self):
        '''Initialize the TV Guide updater'''
        self.guide_updater = GuideUpdater(self.message_bus)
        guide_dict = { MessageType.UPDATE_GUIDE : MessagePriority.VERY_HIGH }
        self.message_bus.registerMessageHandler(self.guide_updater, guide_dict)
        self.logger.debug("Guide Updater intialized successfully")

    def initialize_media_cache_manager(self):
        '''Initialize the media cache manager'''
        self.media_manager = MediaCacheManager(self.configuration)
        media_dict = {
            MessageType.CONTENT_CONF_UPDATED : MessagePriority.VERY_LOW,
            MessageType.REBUILD_IMAGE_CACHE : MessagePriority.HIGH,
            MessageType.REBUILD_MUSIC_CACHE : MessagePriority.HIGH,
            MessageType.REBUILD_VIDEO_CACHE : MessagePriority.HIGH
            }
        self.message_bus.registerMessageHandler(self.media_manager, media_dict)
        self.logger.debug("Media Manager intialized successfully")

    def quitBackend(self):
        '''Close the backend server'''
        self.message_bus.unregisterAllMessageHandlers()



