'''VideoMetadataSearch - Search video file metadata from IMDB database'''
# pylint: disable-msg=C0301

__licence__ = "GPLv2"
__copyright__ = "2008, Lauri Taimila"
__author__ = "Lauri Taimila <lauri@taimila.com>"

import os
import imdb
import re
import urllib
import threading
from pysqlite2 import dbapi2 as sqlite

from utils.logger import Logger
from utils.configuration import Configuration

class VideoMetadataSearch(threading.Thread):
    """
    Search video file metadata from IMDB database. This class is a thread that
    connects to IMDb website and tries to find information for video file. This
    search finds movies and TV-Series.
    """

    # Video cache database
    __VIDEO_DB = os.path.expanduser(
        Configuration().get_cfg_path() + 'cache/video.db')

    # Video cover art directory
    __ART_DIRECTORY = os.path.expanduser(
        Configuration().get_cfg_path() + 'cache/movie_art')

    # Title split keywords
    __TITLE_SPLIT_KEYWORDS = [
        "[", "]", "~", "(", ")", "dvdscr", "dvdrip", "dvd-rip", "dvdr", "vcd",
        "divx", "xvid", "ac3", "r5", "pal", "readnfo", "uncut", "cd1", "cd2",
        "dvdiso"
        ]

    # Title strip items
    __TITLE_STRIP_SEARCH = [".", "-", "_"]

    def __init__(self, filename):
        """
        Initialize metadata search thread.
        @param filename: Filename as string (find metadata for this file)
        """
        threading.Thread.__init__(self)
        self.setName("Video metadata search thread")
        self.logger = Logger().getLogger(
            'backend.components.mediacache.VideoMetadataSearch')
        self.filename = filename
        self.title, self.season, self.episode = self._parse_filename(filename)
        try:
            self.IMDb = imdb.IMDb()
        except imdb.IMDbError:
            raise IOError("Couldn't connect to IMDB server!")

    def _parse_filename(self, filename):
        """
        Parse filename. Tries to get title of the movie/tv-series etc.

        TV-episode filenames should be format:
            Prison_Break_s02e10_something.something.avi
            Prison.Break - 02x10 - something_something.avi
            etc.

        Movies should be something like:
            movie_name_keyword_something.avi
            movie.name.avi
            etc.
        @param filename: Path of the movie
        @return: Title, season and episode
        """
        season = 0
        episode = 0

        # lowercase filename and remove the path and extension
        filename = filename.lower()
        filename = os.path.split(filename)[1]
        filename = os.path.splitext(filename)[0]

        # strip ., - and _ from filename
        for item in self.__TITLE_STRIP_SEARCH:
            filename = filename.replace(item, ' ')

        # split title at keywords
        for item in self.__TITLE_SPLIT_KEYWORDS:
            filename = filename.split('%s' % item)[0]
        filename = filename.strip()

        # try to get the title, season and episode from filename
        regexp = r'(?P<title>.*?)(s?)(?P<season>\d{1,2})(x|e|xe)(?P<episode>\d{1,2})'
        match = re.search(regexp, filename)
        if not match:
            regexp = r'(?P<title>.*?)season\s(?P<season>\d{1,2})\sepisode\s(?P<episode>\d{1,2})'
            match = re.search(regexp, filename)
        if match:
            filename = match.group('title')
            filename = filename.strip()
            season = match.group('season')
            episode = match.group('episode')

        return filename, int(season), int(episode)

    def run(self):
        """
        Search metadata from IMDB and update video cache database.
        """
        search_results = []
        try:
            search_results = self.IMDb.search_movie(self.title)
        except:
            self.logger.error("IMDB search failed")
            return # Network error or too many results to handle

        if len(search_results) == 0:
            return # No matches for this search

        if search_results[0]['kind'] == "movie":
            # We trust that the first search result is the best
            movie = search_results[0]
            try:
                self.IMDb.update(movie)
            except:
                self.logger.error(
                    "Metadata search failed for: " + self.filename)
                return

            video_type = "MOVIE"
            try:
                title = movie['title']
                year = movie['year']
                # convert to 5-stars rating
                rating = round(float(movie['rating']) / 2)

                genres = ','.join(movie['genres'])
                try:
                    plot_outline = movie['plot outline']
                except:
                    plot_outline = ""

                try:
                    plot_string = movie['plot'][0]
                    plot = plot_string[plot_string.rfind("::")+2:].lstrip()
                except:
                    plot = "Plot not available"

                # IMDb returns sometimes list and sometimes string
                if(type(plot)) is list:
                    plot = plot[0]
                if(type(plot_outline)) is list:
                    plot_outline = plot_outline[0]

                try:
                    runtime = movie['runtime'][0]
                    int(runtime) # Raises exception if not integer
                except:
                    runtime = 0
                p = self._get_persons(movie)
                row = (video_type, title, "", runtime, genres, rating,
                       year, plot_outline, plot, 0, 0, p[0], p[1], p[2],
                       p[3], p[4], p[5], p[6], p[7], p[8], self.filename)
                self._update_video_cache(row)

                # Download and save cover art
                try:
                    self._download_cover_art(movie['cover url'], title)
                except:
                    pass
            except:
                self.logger.error("Couldn't find metadata from IMDB for: " +
                                    self.filename)
                return

        elif search_results[0]['kind'] == "tv series":
            series = search_results[0]
            # We trust that the first search result is the best
            try:
                self.IMDb.update(series)
                self.IMDb.update(series, "episodes")
            except:
                self.logger.error("Couldn't find metadata from IMDB for: " +
                                    self.filename)
                return

            video_type = "TV-SERIES"
            p = self._get_persons(series)
            rating = 0
            #round(float(series['rating']) / 2)
            # convert to 5-stars rating

            series_title = series['title']
            genres = ','.join(series['genres'])
            try:
                time = series['runtime']
                runtime = time[0][:time[0].find(":")]
                int(runtime) # This raises exception if runtime is not integer
            except:
                runtime = 0
            try:
                year = series['series years']
            except:
                year = series['year']
            try:
                title = series['episodes'][self.season][self.episode]['title']
            except:
                title = str(self.episode) + ". Episode"
            try:
                plot =  series['episodes'][self.season][self.episode]['plot']
            except:
                plot = "No plot outline available"
            row = (video_type, title, series_title, runtime, genres, rating, \
                   year, "", plot, self.season, self.episode, p[0], p[1], \
                   p[2], p[3], p[4], p[5] ,p[6], p[7], p[8], self.filename)
            self._update_video_cache(row)

            # Download and save cover art
            try:
                self._download_cover_art(series['cover url'], series_title)
            except:
                pass
        else:
            # This file wasn't identified to be a movie or a TV-series episode.
            return

    def _update_video_cache(self, db_row):
        """
        Update data to video cache database
        @param db_row: List that contains all information we want to store into
                       cache
        """
        #print "Update metadata for " + db_row[1] #FOR DEBUGGING
        try:
            db_conn = sqlite.connect(self.__VIDEO_DB)
            db_cursor = db_conn.cursor()
            db_cursor.execute("""UPDATE metadata
                                 SET type=?,
                                     title=?,
                                     series_title=?,
                                     runtime=?,
                                     genres=?,
                                     rating=?,
                                     year=?,
                                     plot_outline=?,
                                     plot=?,
                                     season=?,
                                     episode=?,
                                     actor_1=?,
                                     actor_2=?,
                                     actor_3=?,
                                     actor_4=?,
                                     actor_5=?,
                                     writer_1=?,
                                     writer_2=?,
                                     director_1=?,
                                     director_2=?
                                 WHERE filename=?""", db_row)
            db_conn.commit()
            db_conn.close()
        except Exception, e:
            self.logger.error("Exception occured during video cache update: " +
                                e.message)


    def _get_persons(self, movie):
        """
        Get a list of persons. First five names are actors, then comes two
        directors and two writers.
        @param movie: Movie name
        @return: List of strings containing actors, directors and writers
        """
        try:
            a1 = movie['actors'][0]['name']
        except:
            a1 = ""
        try:
            a2 = movie['actors'][1]['name']
        except:
            a2 = ""
        try:
            a3 = movie['actors'][2]['name']
        except:
            a3 = ""
        try:
            a4 = movie['actors'][3]['name']
        except:
            a4 = ""
        try:
            a5 = movie['actors'][4]['name']
        except:
            a5 = ""
        try:
            w1 = movie['writer'][0]['name']
        except:
            w1 = ""
        try:
            w2 = movie['writer'][1]['name']
        except:
            w2 = ""
        try:
            d1 = movie['director'][0]['name']
        except:
            d1 = ""
        try:
            d2 = movie['director'][1]['name']
        except:
            d2 = ""

        return [a1, a2, a3, a4, a5, w1, w2, d1, d2]

    def _download_cover_art(self, url, title):
        """
        Download cover art from given URL and save it to cache
        @param url: URL to the image file
        @param title: Title of the series or movie
        """
        # Check if we have cover art for this series or movie already
        if not os.path.exists(
            os.path.join(self.__ART_DIRECTORY, title + ".jpg")):
            try:
                image = urllib.urlopen(url)
            except:
                return
            dest = open(os.path.join(self.__ART_DIRECTORY, title + ".jpg"),'w')
            dest.write(image.read())
            dest.close()
