'''WeatherManager - Download local weather conditions from the net.'''

__licence__ = "GPLv2"
__copyright__ = "2007, Lauri Taimila"
__author__ = "Lauri Taimila <lauri@taimila.com>"

import os
from pysqlite2 import dbapi2 as sqlite
from backend.components.weather.weather_fetcher import WeatherFetcher

# Configuration
from utils.configuration import Configuration

# Messaging system
from backend.core.message_type_priority import MessageType
from backend.core.message_handler import MessageHandler

from utils.logger import Logger

class WeatherManager(MessageHandler):
    """
    WeatherManager is a backend component that downloades local weather
    conditions from the internet. Downloaded weather information is stored
    in weather cache (SQLite database). Weather conditions are updated when
    WeatherFetcher receives UPDATE_WEATHER message from the messagebus.
    """

    # Feed cache database file
    WEATHER_DB = os.path.expanduser(
        Configuration().get_cfg_path() + 'cache/weather.db')

    def __init__(self, message_bus, configuration):
        """
        Create a new WeatherFetcher object. Only one object should be created.
        @param message_bus: Backend messagebus object
        @param configuration: Configuration object
        """
        MessageHandler.__init__(self)
        self.message_bus = message_bus
        self.logger = Logger().getLogger(
                                   'backend.components.weather.WeatherManager')
        self.location_information = ""
        #TODO: Set correct value from configuration
        if not os.path.exists(self.WEATHER_DB):
            self.createWeatherCacheDatabase()

    def updateFeedCache(self):
        """
        Update all feeds to cache in a new thread and after
        that emit FEED_DB_UPDATED message.
        """
        fetch_thread = WeatherFetcher(self.message_bus,
                                      self.location_infromation)
        fetch_thread.start()

    def createWeatherCacheDatabase(self):
        """
        Create a weather cache database file and tables.
        """
        db_conn = sqlite.connect(self.WEATHER_DB)
        db_cursor = db_conn.cursor()
        #TODO: Define table. What information we want to store?
        db_cursor.execute("""CREATE TABLE weather(
                                          temperature TEXT,
                                          time TIME,
                                          date DATE,
                                          PRIMARY KEY(time,date))""")
        db_conn.commit()
        db_conn.close()

    # Implementes MessageHandler interface
    def handleMessage(self, message):
        '''Handles messages passed to it'''
        if message.get_type() == MessageType.UPDATE_WEATHER:
            self.updateWeatherCache()

    def getName(self):
        '''Returns the name of the message handler'''
        return "WeatherFetcher"


