'''Entertainer Frontend - This is a client side of Entertainer.'''

__licence__ = "GPLv2"
__copyright__ = "2007, Lauri Taimila"
__author__ = "Lauri Taimila <lauri@taimila.com>"
__version__ = "0.1"

import sys

from utils.theme import Theme
from utils.configuration import Configuration
from utils.logger import Logger
from frontend.backend_connection import BackendConnection

# Media libraries
from frontend.medialibrary.feeds import FeedLibrary
from frontend.medialibrary.music import MusicLibrary
from frontend.medialibrary.images import ImageLibrary
from frontend.medialibrary.videos import VideoLibrary

# GUI
# import gtk    Currently not used
import clutter
import gobject
from frontend.gui.user_interface import UserInterface

class FrontendClient:
    """
    Entertainer frontend

    This is a frontend application of the Entertainer. Frontend is a GUI part
    that user sees on the screen. This class is a core of the frontend. Frontend
    connects to the backend's messagebus at startup.
    """

    def __init__(self, from_tray_icon=False):
        """
        Create a new frontend.

        This initializes all frontend stuff like media librarys, remote
        control receiver and GUI. After this we just wait user actions.
        """

        #Check that installed clutter version is series 0.5 or higher
        if int(clutter.__version__[2]) < 5:
            print """It seems that your Clutter version is too old. Entertainer
                     requires Clutter 0.5 or later. Execution Aborted!"""
            sys.exit(1)

        self.configuration = Configuration()
        self.logger = Logger().getLogger('frontend.FrontendClient')
        self.theme = Theme()
        self.backend_connection = None    # Connection to the backend server
        self.feed_library = None          # Feed library
        self.music_library = None         # Music library
        self.image_library = None         # Image library
        self.video_library = None         # Video library
        self.ui = None                    # User interface of the frontend
        self.from_tray_icon = from_tray_icon

        # Initialize clutter threading system.
        #gtk.gdk.threads_init()
        gobject.threads_init()
        clutter.threads_init()
        #clutter.init()

        self.initialize_backend_connection()
        self.initialize_media_library()
        self.initialize_gui()
        self.ui.start_up(self.configuration.start_in_fullscreen())
        if(not self.from_tray_icon):
            clutter.main()
        #gtk.main()

    def initialize_media_library(self):
        """Initialize media library"""
        self.feed_library = FeedLibrary(self.backend_connection)
        self.music_library = MusicLibrary(self.backend_connection)
        self.image_library = ImageLibrary(self.backend_connection)
        self.video_library = VideoLibrary(self.backend_connection)

    def initialize_backend_connection(self):
        """Connect to the backend server."""
        try:
            self.backend_connection = BackendConnection()
            self.logger.debug("Connected to the Entertainer backend server.")
        except:
            print "Couldn't connect to the backend server. Execution aborted!"
            print "Make sure that Entertainer backend server is up and running."
            sys.exit(1)

    def initialize_gui(self):
        """
        Initialize frontend GUI.

        Create a gui and dislpay it on the screen.
        """
        self.ui = UserInterface(self)

    def quit_frontend(self):
        """
        Quit Frontend

        This is called when user closes frontend. Before closing application
        the backend connection etc. are closed in a clean way."""
        try:
            self.backend_connection.close_connection()
        except Exception, e:
            print "Error closing Backend Connection:", e.message
            print "Closing anyway"
            sys.exit(0)

        #gtk.main_quit()
        if(not self.from_tray_icon):
            clutter.main_quit()
        sys.exit(0)

    def getTheme(self):
        """
        Get frontend's theme
        @return Theme object - Currently used theme
        """
        return self.theme

    def get_configuration(self):
        """
        Get configuration
        @return Configuration object
        """
        return self.configuration

    def get_backend_connection(self):
        """
        Get Backend connection
        @return BackendConnection - active connection to the Entertainer backend
        """
        return self.backend_connection

    def get_user_interface(self):
        """
        Get interface object.
        @return UserInterface object
        """
        return self.ui

    def get_feed_library(self):
        """Get feed library"""
        return self.feed_library

    def get_image_library(self):
        """Get image library"""
        return self.image_library

    def get_music_library(self):
        """Get music library"""
        return self.music_library

    def get_video_library(self):
        """Get video library"""
        return self.video_library
