'''AudioPlayScreen - Screen displays information of currently playing audio'''

__licence__ = "GPLv2"
__copyright__ = "2007, Lauri Taimila"
__author__ = "Lauri Taimila <lauri@taimila.com>"

import gtk
import pango
import gobject
import clutter

from frontend.user_event import UserEvent
from frontend.gui.screens.screen import Screen

from frontend.gui.widgets.scroll_area import ScrollArea
from frontend.gui.widgets.progress_bar import ProgressBar
from frontend.gui.widgets.eyecandy_texture import EyeCandyTexture
from frontend.gui.widgets.loading_animation import LoadingAnimation

from frontend.gui.tabs.tab import Tab
from frontend.gui.widgets.tab_group import TabGroup

#from frontend.gui.widgets.justified_text_menu_item import (
#    JustifiedTextMenuItem)

#from frontend.gui.widgets.list_indicator import ListIndicator

class AudioPlayScreen(Screen):
    """
    AudioPlayScreen

    Screen that displays information of currently playing audio track.
    """

    def __init__(self, ui, frontend, track):
        """
        Initialize screen

        @param ui: UserInterface object
        @param frontend: FrontendClient object
        @param track: Track object (We display information of this track)
        """
        self.group = clutter.Group()
        Screen.__init__(self, ui.STAGE_WIDTH, ui.STAGE_HEIGHT)

        self.ui = ui
        self.frontend = frontend
        self.player = ui.get_media_player()
        self.theme = frontend.getTheme()
        self.library = self.frontend.get_music_library()
        self.track = track
        self.album = track.get_album()

        # Create album art (this is displayed on all tab pages
        if(self.album.has_album_art()):
            pixbuf = gtk.gdk.pixbuf_new_from_file(
                self.album.get_album_art_url())
        else:
            pixbuf = gtk.gdk.pixbuf_new_from_file(
                self.theme.getImage("default_album_art"))
        self.art = EyeCandyTexture(pixbuf, self.ui.x(0.3148),
            self.ui.y(0.5599))
        self.art.set_position(self.ui.get_abs_x_pos(0.1),
            self.ui.get_abs_y_pos(0.22))
        self.art.set_rotation(clutter.Y_AXIS, 25, 0, 0, 0)
        self.art.show()
        self.group.add(self.art)

        # Tabs
        self.tab_group = TabGroup(0.95, 0.13, self.theme.getColor("title"))
        self.progress_bar = None
        self.track_length = None
        self.current_pos = None
        self.artist_label = None
        self.album_label = None
        self.track_label = None
        tab1 = self._create_currently_playing_tab()
        tab2 = self._create_lyrics_tab()
        tab3 = self._create_playlist_tab()
        self.tab_group.add_tab(tab1)
        self.tab_group.add_tab(tab2)
        self.tab_group.add_tab(tab3)
        self.tab_group.set_y(self.ui.y(0.1))
        self.tab_group.set_anchor_point_from_gravity(clutter.GRAVITY_CENTER)
        self.tab_group.set_x(self.ui.x(0.5))
        self.tab_group.show()
        self.tab_group.set_active(True)
        self.group.add(self.tab_group)

        self.group.add(tab1)
        self.group.add(tab2)
        self.group.add(tab3)

        self.group.show()
        ui.getStage().add(self.group)

        self.ui.get_media_player().set_media(self.track)
        self.ui.get_media_player().play()
        gobject.timeout_add(250, self._update_progress_bar) # 1/4 sec

    def _create_currently_playing_tab(self):
        """
        Create a tab that shows information on currenlty playing audio track.
        """
        currently_playing_tab = Tab(self.frontend, "Currently playing")

        # Track name
        self.track_label = clutter.Label()
        self.track_label.set_name("track_label")
        self.track_label.set_font_name(self.theme.getFont() + " " + \
            str(self.ui.y(0.04948)) + "px")
        self.track_label.set_color(self.theme.getColor("text"))
        if self.track.get_tracknumber() == 0:
            self.track_label.set_text(str(self.track.get_title()))
        else:
            self.track_label.set_text(str(self.track.get_tracknumber()) + \
                ". " + self.track.get_title())
        self.track_label.set_size(self.ui.x(0.4978), self.ui.y(0.06510))
        self.track_label.set_position(self.ui.x(0.5), self.ui.y(0.33))
        self.track_label.set_ellipsize(pango.ELLIPSIZE_END)
        self.track_label.set_line_wrap(False)
        self.track_label.show()
        currently_playing_tab.add(self.track_label)

        # Album name
        self.album_label = clutter.Label()
        self.album_label.set_name("album_label")
        self.album_label.set_font_name(self.theme.getFont() + " " + \
            str(self.ui.y(0.0416)) + "px")
        self.album_label.set_color(self.theme.getColor("subtitle"))
        if self.album.get_year() != 0:
            self.album_label.set_text("From " + self.album.get_title() + \
                ", " + str(self.album.get_year()))
        else:
            self.album_label.set_text("From " + self.album.get_title())
        self.album_label.set_size(self.ui.x(0.4978), self.ui.y(0.06510))
        self.album_label.set_position(self.ui.x(0.5), self.ui.y(0.43))
        self.album_label.set_ellipsize(pango.ELLIPSIZE_END)
        self.album_label.set_line_wrap(False)
        self.album_label.show()
        currently_playing_tab.add(self.album_label)

        # Artist name
        self.artist_label = clutter.Label()
        self.artist_label.set_name("artist_label")
        self.artist_label.set_font_name(self.theme.getFont() + " " + \
            str(self.ui.y(0.0416)) + "px")
        self.artist_label.set_color(self.theme.getColor("subtitle"))
        self.artist_label.set_text("By " + self.track.get_artist())
        self.artist_label.set_size(self.ui.x(0.4978), self.ui.y(0.06510))
        self.artist_label.set_position(self.ui.x(0.5), self.ui.y(0.53))
        self.artist_label.set_ellipsize(pango.ELLIPSIZE_END)
        self.artist_label.set_line_wrap(False)
        self.artist_label.show()
        currently_playing_tab.add(self.artist_label)

        # Current position as time stamp
        self.current_pos = clutter.Label()
        self.current_pos.set_name("current_pos")
        self.current_pos.set_font_name(self.theme.getFont() + " " + \
            str(self.ui.y(0.0365)) + "px")
        self.current_pos.set_color(self.theme.getColor("title"))
        self.current_pos.set_text("00:00")
        self.current_pos.set_position(self.ui.x(0.5), self.ui.y(0.665))
        self.current_pos.show()
        currently_playing_tab.add(self.current_pos)

        self.progress_bar = ProgressBar(self.ui.x(0.3001), self.ui.y(0.03906),
            self.theme.getColor("title"))
        self.progress_bar.set_name("progress_bar")
        self.progress_bar.set_position(self.ui.x(0.561), self.ui.y(0.667))
        self.progress_bar.show()
        currently_playing_tab.add(self.progress_bar)

        self.track_length = clutter.Label()
        self.track_length.set_name("track_length")
        self.track_length.set_font_name(self.theme.getFont() + " " + \
            str(self.ui.y(0.0365)) + "px")
        self.track_length.set_color(self.theme.getColor("title"))
        self.track_length.set_text(self.track.get_length_string())
        self.track_length.set_position(self.ui.x(0.8712), self.ui.y(0.665))
        self.track_length.show()
        currently_playing_tab.add(self.track_length)

        return currently_playing_tab

    def _update_progress_bar(self):
        """
        Update progress bar
        """
        progress_bar = self.tab_group.get_tab_by_index(0).find_child_by_name(
            "progress_bar")
        current_pos = self.tab_group.get_tab_by_index(0).find_child_by_name(
            "current_pos")
        progress_bar.set_progress(self.player.get_media_position())
        current_pos.set_text(self.player.get_media_position_string())
        return True

    def _create_lyrics_tab(self):
        """
        Create a tab that displays track lyrics. This tab displays loading
        animation until lyrics are available. Lyrics are searched from the
        Internet if local lyrics doesn't exist.
        """
        lyrics_tab = Tab(self.frontend, "Lyrics")

        if self.track.has_lyrics():
            lyrics = clutter.Label()
            lyrics.set_name("lyrics")
            lyrics.set_font_name(self.theme.getFont() + " " + \
                str(self.ui.y(0.0365)) + "px")
            lyrics.set_color(self.theme.getColor("subtitle"))
            lyrics.set_text(self.track.get_lyrics())
            lyrics.set_line_wrap_mode(pango.WRAP_WORD)
            lyrics.set_width(self.ui.x(0.366))

            lyrics_area = ScrollArea(self.ui.x(0.4), self.ui.y(0.57),
                self.theme)
            lyrics_area.set_name("lyrics_area")
            lyrics_area.set_step_size(self.ui.y(0.03906))
            lyrics_area.set_content(lyrics)
            lyrics_area.set_position(self.ui.x(0.5), self.ui.y(0.23))
            lyrics_area.show()
            lyrics_tab.add(lyrics_area)
        else:
            # Display throbber animation while searching for lyrics
            white = clutter.Color(255, 255, 255, 255)
            gray = clutter.Color(128, 128, 128, 128)
            throbber = LoadingAnimation(50, white, gray)
            throbber.set_name("throbber")
            throbber.set_position(self.ui.x(0.7), self.ui.y(0.5))
            throbber.show()
            lyrics_tab.add(throbber)
            self.track.fetch_lyrics(self._lyrics_search_callback)

        return lyrics_tab

    def _lyrics_search_callback(self, lyrics_text):
        """
        This function is called when lyrics search is over.
        @param lyrics: Lyrics or None if lyrics weren't found
        """
        #TODO: We could smoothly fade away throbber
        lyrics_tab = self.tab_group.get_tab_by_index(1)
        lyrics_tab.find_child_by_name("throbber").hide()

        if lyrics_text is None:
            no_lyrics = clutter.Label()
            no_lyrics.set_name("no_lyrics")
            no_lyrics.set_font_name(self.theme.getFont() + " " + \
                str(self.ui.y(0.0365)) + "px")
            no_lyrics.set_color(self.theme.getColor("title"))
            no_lyrics.set_text("No lyrics for this track")
            no_lyrics.set_anchor_point_from_gravity(clutter.GRAVITY_CENTER)
            no_lyrics.set_position(self.ui.x(0.7), self.ui.y(0.5))
            no_lyrics.show()
            lyrics_tab.add(no_lyrics)
        else:
            lyrics = clutter.Label()
            lyrics.set_font_name(self.theme.getFont() + " " + \
                str(self.ui.y(0.0365)) + "px")
            lyrics.set_color(self.theme.getColor("subtitle"))
            lyrics.set_text(lyrics_text)
            lyrics.set_line_wrap_mode(pango.WRAP_WORD)
            lyrics.set_width(self.ui.x(0.366))

            lyrics_area = ScrollArea(self.ui.x(0.4), self.ui.y(0.57),
                self.theme)
            lyrics_area.set_name("lyrics_area")
            lyrics_area.set_step_size(self.ui.y(0.03906))
            lyrics_area.set_content(lyrics)
            lyrics_area.set_position(self.ui.x(0.5), self.ui.y(0.23))
            lyrics_area.show()
            lyrics_tab.add(lyrics_area)
            self.library.save_lyrics(self.track, lyrics_text)

    def _create_playlist_tab(self):
        """
        Create a tab that displays tracks on current playlist.
        """
        playlist_tab = Tab(self.frontend, "Current playlist")
        return playlist_tab

    def update(self, track):
        """
        This is called when track changes.
        @param track: Currently playing track
        """
        self.track = track
        self.album = track.get_album()

        if self.track.get_tracknumber() == 0:
            self.track_label.set_text(str(self.track.get_title()))
        else:
            self.track_label.set_text(str(self.track.get_tracknumber()) + \
                ". " + self.track.get_title())

        if self.album.get_year() != 0:
            self.album_label.set_text("From " + self.album.get_title() + \
                ", " + str(self.album.get_year()))
        else:
            self.album_label.set_text("From " + self.album.get_title())        
            self.artist_label.set_text("By " + self.track.get_artist())

        self.current_pos.set_text("00:00")
        self.track_length.set_text(self.track.get_length_string())
        self.progress_bar.set_progress(0)

    def get_group(self):
        """
        Get all actors of the screen.
        @return clutter.Group
        """
        return self.group

    def get_type(self):
        """Return screen type."""
        return Screen.NORMAL

    def get_name(self):
        """Return screen name (human readble)"""
        return "AudioPlayScreen"

    def handle_user_event(self, event):
        """
        Handle screen specific user events
        @param event: UserEvent object
        """
        event_type = event.get_type()

        if event_type == UserEvent.NAVIGATE_UP:
            tab = self.tab_group.get_current_tab()

            # If we are on Lyrics tab
            if tab.get_title() == "Lyrics" and not self.tab_group.is_active():
                area = tab.find_child_by_name("lyrics_area")
                if area.is_active() and area.get_offset() == 0:
                    area.set_active(False)
                    self.tab_group.set_active(True)
                elif area.is_active():
                    area.scroll_up()
        elif event_type == UserEvent.NAVIGATE_DOWN:
            tab = self.tab_group.get_current_tab()

            # If we are on Lyrics tab
            if tab.get_title() == "Lyrics":
                area = tab.find_child_by_name("lyrics_area")
                if self.tab_group.is_active():
                    self.tab_group.set_active(False)
                    area = tab.find_child_by_name("lyrics_area")
                    try:
                        area.set_active(True)
                    except:
                        # There is no lyrics area since we are still searching
                        # for lyrics
                        pass
                else:
                    area.scroll_down()
        elif event_type == UserEvent.NAVIGATE_LEFT:
            if self.tab_group.is_active():
                self.tab_group.handle_user_event(event)
        elif event_type == UserEvent.NAVIGATE_RIGHT:
            if self.tab_group.is_active():
                self.tab_group.handle_user_event(event)
        elif event_type == UserEvent.NAVIGATE_SELECT:
            pass


