'''Screen - Screen interface. All screens has to implement this interface!'''

__licence__ = "GPLv2"
__copyright__ = "2007, Lauri Taimila"
__author__ = "Lauri Taimila <lauri@taimila.com>"

import clutter

class Screen(object):
    """
    Screen interface

    Screen represents a view. View includes all the widgets on the screen.
    There can be for example Photographs view, Music view, Feed view etc.
    """

    # Screen type constants
    NORMAL = 0
    DIALOG = 1
    OSD = 2

    def __init__(self, width, height):
        """
        You should never create Screen object! This init is supposed to be
        called from child classes that inherit this class.
        """
        rect = clutter.Rectangle()
        rect.set_size(width, height)
        rect.set_color(clutter.Color(0, 0, 0, 0))
        rect.show()
        self.get_group().add(rect)

    def update(self):
        """
        Update screen widgets. This is called always when screen is poped from
        the screen history. Not all widgets need update. For performance
        reasons you should only update those widgets that might have been
        changed. For example main menu may or may not contain "Currently
        Playing" menuitem.
        """
        pass

    def get_type(self):
        """
        Get screen type: Screen.NORMAL, Screen.OSD or Screen.DIALOG
        @return: Integer
        """
        return None

    def get_name(self):
        """
        Get screen name (human readble)
        @return: String
        """
        return "not implemented"

    def get_group(self):
        """
        Get all actors of the Screen in clutter.Group
        @return: clutter.Group
        """
        return None

    def is_interested_in_play_action(self):
        """
        See if screen wants to handle Play action itself. If inheriting screen
        wants to handle play action by user it should override this method to
        return True. This kind of behaviour is required, for exmaple, in music
        library. When user press Play action over certain album we want to play
        that album, not pause current playback.
        @return: boolean. True if screen wants to handle play action
        """
        return False

    def execute_play_action(self):
        """
        This function executes screen spesific play action. For example adds
        album to playlist and starts playback. This function should be
        overriden in inheriting class.
        """
        pass

    def handle_user_event(self, event):
        """
        Handle user event. This is called for current screen when user event
        occurs. This way current screen can define how to react.
        @param event: UserEvent object
        """
        pass

