'''VideoOSDScreen - Screen is used for video playback OSD'''

__licence__ = "GPLv2"
__copyright__ = "2007, Lauri Taimila"
__author__ = "Lauri Taimila <lauri@taimila.com>"

import gtk
import gobject
import clutter

from frontend.user_event import UserEvent
from frontend.gui.screens.screen import Screen
from frontend.gui.widgets.progress_bar import ProgressBar

class VideoOSDScreen(Screen):
    """
    VideoOSDScreen

    This screen is displayed when video is being watched. Usually this screen
    doesn't have any visible elements. User actions such as pause and rewind
    are displayed as on-screen-graphics.
    """

    def __init__(self, user_interface, theme, player):
        """
        Initialize screen

        @param user_interface: UserInterface object
        @param theme: theme object
        @param player: MediaPlayer object
        """
        self.group = clutter.Group()
        Screen.__init__(self, user_interface.STAGE_WIDTH,
            user_interface.STAGE_HEIGHT)

        self.user_interface = user_interface
        self.theme = theme
        self.player = player

        self.progress_bar = None
        self.current_pos = None
        self.aspect_textures = None
        self.timeout_key = None
        self.progress_bar_timeout_key = None

        self._create_aspect_ratio_textures()

        self.user_interface.getStage().add(self.group)

        self._create_progress_bar()
        gobject.timeout_add(250, self._update_progress_bar) # 1/4 sec

    def _create_progress_bar(self):
        """
        Create the progress bar
        @author Joshua Scotton
        """
        # Current position as time stamp
        self.current_pos = clutter.Label()
        self.current_pos.set_name("current_pos")
        self.current_pos.set_font_name(self.theme.getFont() + " " + \
            str(self.user_interface.y(0.0365)) + "px")
        self.current_pos.set_color(self.theme.getColor("title"))
        self.current_pos.set_text("00:00")
        self.current_pos.set_position(self.user_interface.x(0.6),
            self.user_interface.y(0.900))
        self.group.add(self.current_pos)

        self.progress_bar = ProgressBar(self.user_interface.x(0.3001),
            self.user_interface.y(0.03906), self.theme.getColor("title"))
        self.progress_bar.set_name("progress_bar")
        self.progress_bar.set_position(self.user_interface.x(0.68),
            self.user_interface.y(0.901))
        self.group.add(self.progress_bar)
        self.display_progress_bar()

    def _update_progress_bar(self):
        """
        Update progress bar
        @author Joshua Scotton
        """
        self.progress_bar.set_progress(self.player.get_media_position())
        self.current_pos.set_text(self.player.get_media_position_string())
        return True

    def display_progress_bar(self, hide_after_delay=True):
        """
        This displays the progress bar for three seconds
        @author Joshua Scotton
        """
        self.progress_bar.show()
        self.current_pos.show()
        if self.progress_bar_timeout_key is not None:
            gobject.source_remove(self.progress_bar_timeout_key)
        if hide_after_delay:
            self.progress_bar_timeout_key = gobject.timeout_add(3000,
                self.hide_progress_bar)

    def hide_progress_bar(self):
        """
        This hides the progress bar
        @author Joshua Scotton
        """
        self.progress_bar.hide()
        self.current_pos.hide()

    def _create_aspect_ratio_textures(self):
        """
        Create textures that which are displayed when user changes aspect ratio.
        """
        #FIXME: We could use Cairo instead of pixmap graphics
        pixbuf1 = gtk.gdk.pixbuf_new_from_file(self.theme.getImage(
            "native_aspect_ratio"))
        pixbuf2 = gtk.gdk.pixbuf_new_from_file(self.theme.getImage(
            "widescreen_aspect_ratio"))
        pixbuf3 = gtk.gdk.pixbuf_new_from_file(self.theme.getImage(
            "zoom_aspect_ratio"))
        pixbuf4 = gtk.gdk.pixbuf_new_from_file(self.theme.getImage(
            "compromise_aspect_ratio"))
        texture_1 = clutter.Texture()
        texture_2 = clutter.Texture()
        texture_3 = clutter.Texture()
        texture_4 = clutter.Texture()
        texture_1.set_pixbuf(pixbuf1)
        texture_2.set_pixbuf(pixbuf2)
        texture_3.set_pixbuf(pixbuf3)
        texture_4.set_pixbuf(pixbuf4)
        self.group.add(texture_1)
        self.group.add(texture_2)
        self.group.add(texture_3)
        self.group.add(texture_4)
        self.aspect_textures = [texture_1, texture_2, texture_3, texture_4]
        self.timeout_key = None # This is used when canceling timeouts
        for texture in self.aspect_textures:
            texture.set_position(
                (self.user_interface.STAGE_WIDTH - texture.get_width()) / 2,
                int(self.user_interface.STAGE_HEIGHT / 1.5))

    def _hide_aspect_ratio_logo(self, number):
        """
        Hide aspect ratio texture. This is a callback function and
        shouldn't be called directly.
        """
        self.aspect_textures[number].hide()
        self.timeout_key = None
        return False

    def _display_aspect_ratio_logo(self, number):
        """
        Display aspect ratio logo on screen when ratio is changed.
        @param number: Number of the aspect ratio
        """
        if self.timeout_key is not None:
            gobject.source_remove(self.timeout_key)
        for texture in self.aspect_textures:
            texture.hide()
        self.aspect_textures[number].show()
        self.timeout_key = gobject.timeout_add(2000,
            self._hide_aspect_ratio_logo, number)

    def get_group(self):
        """
        Get all actors of the screen.
        @return: clutter.Group
        """
        return self.group

    def get_type(self):
        """
        Get screen type.
        @return: Integer (See Screen class constants)
        """
        return Screen.OSD

    def get_name(self):
        """
        Get screen name (human readble)
        @return: string
        """
        return "VideoOSD"

    def handle_user_event(self, event):
        """
        Handle screen specific user events
        @param event: UserEvent object
        """
        event_type = event.get_type()

        if event_type == UserEvent.PLAYER_STOP:
            self.display_progress_bar(hide_after_delay=False)
            #FIXME
            print "CREATE VIDEO STOPPED SCREEN HERE"

        # Video aspect ratio changed. Display aspect ratio logo on screen
        elif event_type == UserEvent.USE_ASPECT_RATIO_1:
            self._display_aspect_ratio_logo(0)
        elif event_type == UserEvent.USE_ASPECT_RATIO_2:
            self._display_aspect_ratio_logo(1)
        elif event_type == UserEvent.USE_ASPECT_RATIO_3:
            self._display_aspect_ratio_logo(2)
        elif event_type == UserEvent.USE_ASPECT_RATIO_4:
            self._display_aspect_ratio_logo(3)
        elif event_type == UserEvent.PLAYER_SKIP_FORWARD:
            self.display_progress_bar()
        elif event_type == UserEvent.PLAYER_SKIP_BACKWARD:
            self.display_progress_bar()
        elif event_type == UserEvent.PLAYER_PLAY_PAUSE:
            if self.player.is_playing():
                self.display_progress_bar()
            else:
                self.display_progress_bar(hide_after_delay=False)
