'''SeriesTab - Group of objects that are displayed on tv-series tab'''

__licence__ = "GPLv2"
__copyright__ = "2008, Lauri Taimila"
__author__ = "Lauri Taimila <lauri@taimila.com>"

import gtk
import clutter

from frontend.user_event import UserEvent
from frontend.gui.transitions.transition import Transition

from frontend.gui.tabs.tab import Tab
from frontend.gui.widgets.image_menu import ImageMenu
from frontend.gui.widgets.list_indicator import ListIndicator
from frontend.gui.widgets.image_menu_item import ImageMenuItem
from frontend.gui.widgets.rounded_texture import RoundedTexture

class SeriesTab(Tab):
    """
    Tab can be used as part of the TabGroup
    
    Tab is a very simple container that contains all the widgets and logic
    of the tab page.
    """
    
    def __init__(self, frontend, title):
        """
        Initialize tab
        @param title: Title of the Tab (string)
        @param frontend: FrontendClient object
        """
        Tab.__init__(self, frontend, title)
        self.library = frontend.get_video_library()
        self.frontend = frontend
        self.ui = frontend.get_user_interface()
        self.theme = frontend.getTheme()
        self.list_indicator = None
        self.serie_info = None
        self.series_menu = None
        self.serie_title = None

        if self.library.get_number_of_tv_series() == 0:
            self._create_empty_library_notice()
        else:
            self._create_series_menu()
    
    def can_activate(self):
        """
        Allow if we have some TV-series indexed.
        """
        if self.library.get_number_of_tv_series() == 0:
            return False
        else:
            return True
            
    def set_active(self, boolean):
        """
        Override method from parent class.
        @param boolean: True to set this tab active, False to set inactive
        """
        Tab.set_active(self, boolean)
        if boolean:
            self._update_series_info(
                self.series_menu.get_current_menuitem().get_userdata())
            self.series_menu.set_active(True)
            self.serie_title.show()
            self.serie_info.show()
            self.list_indicator.show()
        else:
            self.series_menu.set_active(False)
            self.serie_title.hide()
            self.serie_info.hide()
            self.list_indicator.hide()
            
    def _create_empty_library_notice(self):
        """
        Create an information box that is displayed if there are no indexed
        tv-series.
        """
        message = "There are no indexed TV-series in Entertainer media "
        message += "library. To add TV-series, click on 'content' button on "
        message += "toolbar and open 'videos' tab. Now click on 'add' button "
        message += "and select some folders which contains video files."
        Tab.show_empty_tab_notice(self, "No TV-series available!", message)

    def _create_series_menu(self):
        """
        Create a view that is displayed when there is indexed TV-series in
        the video library.
        """
        #Create TV-series menu
        self.series_menu = ImageMenu(self.theme)
        self.series_menu.set_item_size(self.ui.x(0.1), self.ui.y(0.25))
        self.series_menu.set_item_spacing(self.ui.y(0.025))
        self.series_menu.set_visible_column_count(2)
        self.series_menu.set_row_count(7)
        self.series_menu.set_orientation(self.series_menu.VERTICAL)
        
        # Default texture
        default_cover = clutter.Texture()
        pix_buffer = gtk.gdk.pixbuf_new_from_file(
            self.theme.getImage("default_movie_art"))
        default_cover.set_pixbuf(pix_buffer)
        
        series = self.library.get_tv_series()
        for serie in series:
            if serie.has_cover_art():
                #texture = clutter.Texture()
                pix_buffer = gtk.gdk.pixbuf_new_from_file(
                    serie.get_cover_art_url())
                #texture.set_pixbuf(pix_buffer)
                texture = RoundedTexture(pix_buffer, self.ui.x(0.1),
                    self.ui.y(0.25))
            else:
                texture = clutter.CloneTexture(default_cover)
            
            item = ImageMenuItem(self.ui.x(0.1), self.ui.y(0.25), texture)
            item.set_userdata(serie)
            self.series_menu.add_actor(item)

        
        self.series_menu.set_position(self.ui.x(0.11), self.ui.y(0.2))
        self.series_menu.set_active(False)
        self.series_menu.show()

        if self.frontend.get_configuration().show_effects():
            self.series_menu.set_animate(True)
        
        self.add(self.series_menu)
    
        # Create list indicator
        self.list_indicator = ListIndicator(0.2, 0.045, self.theme,
            ListIndicator.VERTICAL)
        self.list_indicator.set_position(self.ui.x(0.75), self.ui.y(0.76))
        self.list_indicator.set_delimiter(" | ")
        self.list_indicator.set_maximum(self.series_menu.get_number_of_items())
        self.list_indicator.set_current(1)
        self.add(self.list_indicator)
        
        # Create information labels
        self.serie_title = clutter.Label()
        self.serie_title.set_text(" ")
        self.serie_title.set_font_name(self.theme.getFont() + " bold " + \
            str(self.ui.y(0.042)) + "px")
        self.serie_title.set_color(self.theme.getColor("title"))
        self.serie_title.set_line_wrap(False)
        #self.serie_title.set_ellipsize(pango.ELLIPSIZE_END)
        self.serie_title.set_position(self.ui.x(0.2), self.ui.y(0.75))
        self.serie_title.set_width(self.ui.x(0.5))
        self.serie_title.show()
        self.add(self.serie_title)
        
        self.serie_info = clutter.Label()
        self.serie_info.set_text(" ")
        self.serie_info.set_font_name(self.theme.getFont() + " " + \
            str(self.ui.y(0.034)) + "px")
        self.serie_info.set_color(self.theme.getColor("subtitle"))
        self.serie_info.set_line_wrap(False)
        #self.serie_info.set_ellipsize(pango.ELLIPSIZE_END)
        self.serie_info.set_position(self.ui.x(0.2), self.ui.y(0.82))
        self.serie_info.set_width(self.ui.x(0.5))
        self.serie_info.show()
        self.add(self.serie_info)

    def _update_series_info(self, serie):
        """
        Update TV-Serie information. Updates clutter labels on TV-series tab
        @param serie: Currently selected TV-Serie (TVSeries object)
        """
        info = str(serie.get_number_of_seasons()) + " Seasons\n" + \
            str(serie.get_number_of_episodes()) + " Episodes"
        self.serie_info.set_text(info)
        self.serie_title.set_text(serie.get_title())

    def handle_user_event(self, event):
        """ 
        Handle screen specific user events
        @param event: UserEvent object
        @return: Boolean, Should control be moved back to tab bar from tab
        """
        event_type = event.get_type()
        
        if event_type == UserEvent.NAVIGATE_UP:
            if self.series_menu.get_current_postition() <= 6:
                return True # Move control back to tab bar
            else:
                self.series_menu.move(self.series_menu.UP)
                self._update_series_info(
                    self.series_menu.get_current_menuitem().get_userdata())
                self.list_indicator.set_current(
                    self.series_menu.get_current_postition() + 1)
        elif event_type == UserEvent.NAVIGATE_DOWN:
            self.series_menu.move(self.series_menu.DOWN)
            self._update_series_info(
                self.series_menu.get_current_menuitem().get_userdata())
            self.list_indicator.set_current(
                self.series_menu.get_current_postition() + 1)
        elif event_type == UserEvent.NAVIGATE_LEFT:
            self.series_menu.move(self.series_menu.LEFT)
            self._update_series_info(
                self.series_menu.get_current_menuitem().get_userdata())
            self.list_indicator.set_current(
                self.series_menu.get_current_postition() + 1)
        elif event_type == UserEvent.NAVIGATE_RIGHT:
            self.series_menu.move(self.series_menu.RIGHT)
            self._update_series_info(
                self.series_menu.get_current_menuitem().get_userdata())
            self.list_indicator.set_current(
                self.series_menu.get_current_postition() + 1)
        elif event_type == UserEvent.NAVIGATE_SELECT:
            serie = self.series_menu.get_current_menuitem().get_userdata()
            self.ui.changeScreen(
                self.ui.createScreen("tv-serie", serie), Transition.FORWARD)
            
        return False


