'''Tests the SystemTrayMessageHandler'''

__licence__ = "GPLv2"
__copyright__ = "2008, Joshua Scotton"
__author__ = "Joshua Scotton <josh@joshuascotton.com>"

import unittest
import sys
import os

sys.path += [os.path.abspath('..')]

from utils.system_tray_icon import SystemTrayMessageHandler
from backend.core.message import Message
from backend.core.message_type_priority import MessageType

DEFAULT_MESSAGE = ""
DEFAULT_TITLE = "Entertainer Media Center"
DEFAULT_LENGTH = 5
DEFAULT_ICON = "dialog-information"
DEFAULT_URGENCY = "LOW"

NORMAL_ICON = 0
RECORD_ICON = 1

class MockMessageBusProxy:
    """This is a dummy MessageBusProxy class for the test"""

    def __init__(self):
        pass

class MockSystemTrayIcon:
    """This is a dummy SystemTrayIcon class for the test"""

    NORMAL_ICON = 0
    RECORD_ICON = 1

    def __init__(self):
        self.current_message = DEFAULT_MESSAGE
        self.current_title = DEFAULT_TITLE
        self.current_length = DEFAULT_LENGTH
        self.current_icon = DEFAULT_ICON
        self.current_urgency = DEFAULT_URGENCY
        self.main_icon = NORMAL_ICON

    def check_current_values(self, message, title, length, icon, urgency):
        '''See SystemTrayIcon'''
        return (message == self.current_message) \
                and (title == self.current_title) \
                and (length == self.current_length) \
                and (icon == self.current_icon) \
                and (urgency == self.current_urgency)

    def show_message(self, message,
                    title=DEFAULT_TITLE,
                    length=DEFAULT_LENGTH, icon=DEFAULT_ICON,
                    urgency=DEFAULT_URGENCY):
        '''See SystemTrayIcon'''
        self.current_message = message
        self.current_title = title
        self.current_length = length
        self.current_icon = icon
        self.current_urgency = urgency

        print (self.current_message, self.current_title, self.current_length,
            self.current_icon, self.current_urgency)

    def change_icon(self, icon_state):
        '''See SystemTrayIcon'''
        if icon_state == NORMAL_ICON:
            self.main_icon = NORMAL_ICON
        elif icon_state == RECORD_ICON:
            self.main_icon = RECORD_ICON

class SystemTrayMessageHandlerTest(unittest.TestCase):
    '''Tests SystemTrayMessageHandler'''

    def setUp(self):
        """Sets up everything for the test"""
        self.tray_icon = MockSystemTrayIcon()
        self.message_handler = SystemTrayMessageHandler(self.tray_icon)

    def testSystemTrayMessageHandler001(self):
        """Check if main_icon is set up right for tests"""
        self.assertEquals(self.message_handler.get_tray_icon().main_icon,
                          NORMAL_ICON)

    def testSystemTrayMessageHandler002(self):
        """Check if current values are set up right for tests"""
        self.assertTrue(
            self.message_handler.get_tray_icon().check_current_values(
                DEFAULT_MESSAGE, DEFAULT_TITLE, DEFAULT_LENGTH,
                DEFAULT_ICON, DEFAULT_URGENCY)
            )

    def testSystemTrayMessageHandler003(self):
        """Test init 001"""
        self.assertFalse(self.message_handler.get_debug())

    def testSystemTrayMessageHandler004(self):
        """Test init 002"""
        self.assertEqual(self.message_handler.get_tray_icon(), self.tray_icon)

    def testSystemTrayMessageHandler005(self):
        """Test init 003"""
        self.assertEqual(self.message_handler.get_message_bus_proxy(), None)

    def testSystemTrayMessageHandler006(self):
        """Test init 004"""
        self.message_handler = SystemTrayMessageHandler(self.tray_icon, True)
        self.assertTrue(self.message_handler.get_debug())

    def testSystemTrayMessageHandler007(self):
        """Test init 005"""
        self.message_handler = SystemTrayMessageHandler(self.tray_icon, True)
        self.assertEqual(self.message_handler.get_tray_icon(), self.tray_icon)

    def testSystemTrayMessageHandler008(self):
        """Test set_tray_icon()"""
        new_tray_icon = MockSystemTrayIcon()
        self.message_handler.set_tray_icon(new_tray_icon)
        self.assertEqual(new_tray_icon, self.message_handler.tray_icon)

    def testSystemTrayMessageHandler009(self):
        """Test init 009"""
        self.assertEqual(self.message_handler.get_tray_icon(), self.tray_icon)

    def testSystemTrayMessageHandler010(self):
        """Test get_debug"""
        self.assertFalse(self.message_handler.get_debug())

    def testSystemTrayMessageHandler011(self):
        """Test set_debug() when setting True"""
        self.message_handler.set_debug(True)
        self.assertTrue(self.message_handler.get_debug())

    def testSystemTrayMessageHandler012(self):
        """Test set_debug() when setting False"""
        self.message_handler.set_debug(True)
        self.message_handler.set_debug(False)
        self.assertFalse(self.message_handler.get_debug())

    def testSystemTrayMessageHandler013(self):
        """Test getName()"""
        self.assertEqual(self.message_handler.getName(),
            "System tray icon message handler")

    def testSystemTrayMessageHandler014(self):
        """Test action_recording_started()"""
        self.message_handler.action_recording_started()
        self.assertTrue(
            self.message_handler.get_tray_icon().check_current_values(
                "Recording started...", DEFAULT_TITLE, DEFAULT_LENGTH,
                DEFAULT_ICON, DEFAULT_URGENCY)
            )
    def testSystemTrayMessageHandler015(self):
        """Test action_recording_started()"""
        self.message_handler.action_recording_started()
        self.assertEquals(self.message_handler.get_tray_icon().main_icon,
                  RECORD_ICON)

    def testSystemTrayMessageHandler016(self):
        """Test action_recording_stopped()"""
        self.message_handler.action_recording_stopped()
        self.assertTrue(
            self.message_handler.get_tray_icon().check_current_values(
                "Recording stopped...", DEFAULT_TITLE, DEFAULT_LENGTH,
                DEFAULT_ICON, DEFAULT_URGENCY)
            )
    def testSystemTrayMessageHandler017(self):
        """Test action_recording_stopped()"""
        self.message_handler.action_recording_started()
        self.message_handler.action_recording_stopped()
        self.assertEquals(self.message_handler.get_tray_icon().main_icon,
                  NORMAL_ICON)
    def testSystemTrayMessageHandler018(self):
        """Test action_recording_conflict()"""
        self.message_handler.action_recording_conflict()
        self.assertTrue(
            self.message_handler.get_tray_icon().check_current_values(
                "Recording Conflict!", DEFAULT_TITLE, DEFAULT_LENGTH,
                "dialog-warning", "NORMAL")
            )
    def testSystemTrayMessageHandler019(self):
        """Test action_recording_conflict()"""
        self.message_handler.action_recording_started()
        self.message_handler.action_recording_conflict()
        self.assertEquals(self.message_handler.get_tray_icon().main_icon,
                  NORMAL_ICON)

    def testSystemTrayMessageHandler020(self):
        """Test handleMessage RECORDING_STARTED"""
        self.message_handler.handleMessage(
            Message(MessageType.RECORDING_STARTED))
        self.assertTrue(
            self.message_handler.get_tray_icon().check_current_values(
                "Recording started...", DEFAULT_TITLE, DEFAULT_LENGTH,
                DEFAULT_ICON, DEFAULT_URGENCY)
            )
    def testSystemTrayMessageHandler021(self):
        """Test handleMessage RECORDING_STARTED"""
        self.message_handler.handleMessage(
            Message(MessageType.RECORDING_STARTED))
        self.assertEquals(self.message_handler.get_tray_icon().main_icon,
                  RECORD_ICON)

    def testSystemTrayMessageHandler022(self):
        """Test handleMessage RECORDING_STOPPED"""
        self.message_handler.handleMessage(
            Message(MessageType.RECORDING_STOPPED))
        self.assertTrue(
            self.message_handler.get_tray_icon().check_current_values(
                "Recording stopped...", DEFAULT_TITLE, DEFAULT_LENGTH,
                DEFAULT_ICON, DEFAULT_URGENCY)
            )
    def testSystemTrayMessageHandler023(self):
        """Test handleMessage RECORDING_STOPPED"""
        self.message_handler.handleMessage(
            Message(MessageType.RECORDING_STARTED))
        self.message_handler.handleMessage(
            Message(MessageType.RECORDING_STOPPED))
        self.assertEquals(self.message_handler.get_tray_icon().main_icon,
                  NORMAL_ICON)

    def testSystemTrayMessageHandler024(self):
        """Test handleMessage RECORDING_CONFLICT"""
        self.message_handler.handleMessage(
            Message(MessageType.RECORDING_CONFLICT))
        self.assertTrue(
            self.message_handler.get_tray_icon().check_current_values(
                "Recording Conflict!", DEFAULT_TITLE, DEFAULT_LENGTH,
                "dialog-warning", "NORMAL")
            )
    def testSystemTrayMessageHandler025(self):
        """Test handleMessage RECORDING_CONFLICT"""
        self.message_handler.handleMessage(
            Message(MessageType.RECORDING_STARTED))
        self.message_handler.handleMessage(
            Message(MessageType.RECORDING_CONFLICT))
        self.assertEquals(self.message_handler.get_tray_icon().main_icon,
                  NORMAL_ICON)

    def testSystemTrayMessageHandler026(self):
        """testSystemTrayMessageHandler026: Test set_message_bus_proxy()"""
        testProxy = MockMessageBusProxy()
        self.message_handler.set_message_bus_proxy(testProxy)
        self.assertEquals(testProxy, self.message_handler.message_bus_proxy)

    def testSystemTrayMessageHandler027(self):
        """testSystemTrayMessageHandler027: Test get_message_bus_proxy()"""
        self.assertEquals(self.message_handler.get_message_bus_proxy(), None)

