'''Configuration - Class represents Entertainer's configuration'''

__licence__ = "GPLv2"
__copyright__ = "2007, Lauri Taimila"
__author__ = "Lauri Taimila <lauri@taimila.com>"

import os
import sys
import shutil
import ConfigParser
from ConfigParser import ParsingError, NoSectionError, NoOptionError

from backend.core.message_type_priority import MessageType
from backend.core.message_handler import MessageHandler

class Configuration(MessageHandler):
    """
    Configuration of the Entertainer

    This class is an interface to all configuration parameters. All components
    of Entertainer should get configuration vales through this class. Object
    from this class reads current config files and returns values based on
    those files. If there are missing lines or other errors in config file,
    then object return default values defined in this class.
    """

    CFG_DIR = "~/.config/entertainer/"

    # Preferences file
    PREFERENCES_CONF = os.path.expanduser(CFG_DIR + 'preferences.conf')

    # Content file
    CONTENT_CONF = os.path.expanduser(CFG_DIR + 'content.conf')

    TRAY_ICON_URL = os.path.expanduser(CFG_DIR + \
        'data/icons/24x24/entertainer.png')

    def __init__(self, cfg_dir=None):
        """
        Read configuration files and setup this object.
        """
        MessageHandler.__init__(self)
        self.cfg_dir = cfg_dir
        if cfg_dir is None:
            self.cfg_dir = self.CFG_DIR

        if not os.path.exists(os.path.expanduser(self.cfg_dir)):
            self.create_cfg_dir()

        self.content_config = ConfigParser.ConfigParser()
        self.preferences = ConfigParser.ConfigParser()
        try:
            self.content_config.readfp(open(self.CONTENT_CONF))
            self.preferences.readfp(open(self.PREFERENCES_CONF))
        except ParsingError:
            print("ParsingError with configuration file.")
            sys.exit(1)
        except IOError:
            print("IOError: Couldn't read configuration file.")
            sys.exit(1)

    def create_cfg_dir(self):
        '''
        Create a configuration directory and default config files.
        '''
        try:
            # Create a list that will contain the parent of the src
            # directory as the first list element
            separator = 'src' + os.sep + 'utils'
            path_list = __file__.split(separator, 1)
            source_config_path = path_list[0] + 'cfg'

            config_path = os.path.expanduser(self.get_cfg_path())

            shutil.copytree(source_config_path, config_path)
        except OSError:
            print "Could not copy cfg directory. Execution aborted..."
            sys.exit(1)

    def update_configuration(self):
        """
        Read configuration files again and update this object.
        """
        try:
            self.content_config.readfp(open(self.CONTENT_CONF))
            self.preferences.readfp(open(self.PREFERENCES_CONF))
        except ParsingError:
            raise Exception("Syntax error in configuration file.")
        except IOError:
            raise IOError("Couldn't read config file.")

    def get_theme_path(self):
        """
        Get absolute path of the used theme.
        @return: string (path)
        """
        theme_path = os.path.expanduser(self.get_cfg_path() + 'themes')
        try:
            theme = self.preferences.get("General", "theme")
        except (NoSectionError, NoOptionError):
            return os.path.join(theme_path, "Default Light")

        if os.path.exists(os.path.join(theme_path, theme)):
            return os.path.join(theme_path, theme)
        else:
            return os.path.join(theme_path, "Default Light")

    def is_notifications_enabled(self):
        """
        Display on-screen notification bubbles on events.
        @return: boolean
        """
        try:
            result = self.preferences.getboolean("General",
                "display_notifications")
        except (NoSectionError, NoOptionError):
            return False
        return result

    def get_feed_fetch_interval(self):
        """
        Get time interval (in minutes) for feed fetching.
        @return: Integer
        """        
        try:
            result = self.content_config.getint("RSS", "fetch_interval")
        except (NoSectionError, NoOptionError):
            return 60 # Default is one hour
        return result

    def get_port(self):
        """
        Get backend server's port number.
        @return: Integer
        """
        try:
            result = self.preferences.getint("General", "backend_port")
        except (NoSectionError, NoOptionError):
            return 45054 # Default port
        return result

    def get_video_folders(self):
        """
        Get list of video folders
        @return:String Array
        """    
        #FIXME: We should prevent ENTERTAINER_DIR/cache/* to be in those folders
        #       This can lead to recursivnes: make thumbs of thumbs of thumbs...
        try:
            folder_list = self.content_config.get("Videos", "folders")
            result = folder_list.split(';')
        except (NoSectionError, NoOptionError):
            return []
        return result

    def get_music_folders(self):
        """
        Get list of music folders
        @return: String Array
        """
        #FIXME: We should prevent ENTERTAINER_DIR/cache/* to be in those folders
        #       This can lead to recursivnes: make thumbs of thumbs of thumbs...
        try:
            music_list = self.content_config.get("Music", "folders")
            result = music_list.split(';')
        except (NoSectionError, NoOptionError):
            return []
        return result

    def get_image_folders(self):
        """
        Get list of image folders
        @return: String Array
        """
        #FIXME: We should prevent ENTERTAINER_DIR/cache/* to be in those folders
        #       This can lead to recursivnes: make thumbs of thumbs of thumbs...
        try:
            image_list = self.content_config.get("Images", "folders")
            result = image_list.split(';')
        except (NoSectionError, NoOptionError):
            return []
        return result

    def get_feeds(self):
        """
        Get list of feeds.
        @return: String Array
        """
        try:
            rss_feeds = self.content_config.get("RSS", "feeds")
            result = rss_feeds.split(';')
        except (NoSectionError, NoOptionError):
            return []
        return result

    def get_weather_location(self):
        """
        Get list of weather location codes.
        @return: List of strings
        """
        try:
            location = self.content_config.get("Weather", "location")
        except (NoSectionError, NoOptionError):
            location = ''
        return location

    def get_weather_metric(self):
        """
        Should we use metric units when displaying weather conditions
        @return: Boolean
        """
        try:
            result = self.content_config.getboolean("Weather", "metric_units")
        except (NoSectionError, NoOptionError):
            return False
        return result

    def display_weather_in_frontend(self):
        """
        Should we display weather in frontend
        @return: Boolean
        """
        try:
            result = self.content_config.getboolean("Weather",
                "display_in_menu")
        except (NoSectionError, NoOptionError):
            return False
        return result

    def display_cd_eject_in_frontend(self):
        """
        Should we display the cd eject button in frontend
        @return: Boolean
        @author: Joshua Scotton
        """
        try:
            result = self.content_config.getboolean("CD",
                "display_eject_in_menu")
        except (NoSectionError, NoOptionError):
            return False
        return result

    def download_video_metadata(self):
        """
        Get True if video metadata should be downloaded, otherwise False
        """
        try:
            result = self.content_config.getboolean("Videos",
                "download_metadata")
        except (NoSectionError, NoOptionError):
            return False
        return result

    def download_album_art(self):
        """
        Download album art from the Internet.
        @return: boolean
        """
        try:
            result = self.content_config.getboolean("Music",
                "download_album_art")
        except (NoSectionError, NoOptionError):
            return False
        return result

    def download_lyrics(self):
        """
        Download song lyrics from the Internet.
        @return: boolean
        """
        try:
            result = self.content_config.getboolean("Music", "download_lyrics")
        except (NoSectionError, NoOptionError):
            return False
        return result

    def show_effects(self):
        """
        Use animations on user interface.
        @return: boolean
        """
        try:
            result = self.preferences.getboolean("General", "show_effects")
        except (NoSectionError, NoOptionError):
            return False
        return result

    def transition_effect(self):
        """
        Which transition effect should be used when switching screens.
        @return: string (name of the effect)
        """
        try:
            result = self.preferences.get("General", "transition_effect")
        except (NoSectionError, NoOptionError):
            return False
        return result

    def start_in_fullscreen(self):
        """
        Return True if frontend should start in Fullscreen mode, otherwise False
        @return: boolean
        """
        try:
            result = self.preferences.getboolean("General",
                "start_in_fullscreen")
        except (NoSectionError, NoOptionError):
            return False
        return result

    def history_size(self):
        """
        Get screen history size. This determines how many screen are kept in
        memory when scrolling.
        @return: Integer
        """
        try:
            result = self.preferences.getint("General", "history_size")
        except (NoSectionError, NoOptionError):
            return 10
        return result

    # Implements MessageHandler interface
    def handleMessage(self, message):
        """
        Handle received messages.
        @param message: Received Message object
        """
        if message.get_type() == MessageType.CONTENT_CONF_UPDATED:
            self.update_configuration()
        elif message.get_type() == MessageType.PREFERENCES_CONF_UPDATED:
            self.update_configuration()

    def getName(self):
        """
        Get name of the MessageHandler
        @return: string 
        """
        return "Configuration"

    def get_cfg_path(self):
        """
        Get the entertainer configuration path.
        @return: strings
        """
        return self.cfg_dir

    def set_cfg_path(self, path):
        """
        Set the entertainer configuration path.
        """
        if os.path.exists(os.path.expanduser(path)):
            self.cfg_dir = path
        else:
            print "Trying to set non-existent configuration directory"

    def get_tray_icon_url(self):
        '''Get the tray icon url'''
        return self.TRAY_ICON_URL


