"""
Weather Class
  Uses google weathers api to get a weather forecast for the next
  4 days.
"""
__licence__ = "GPLv2"
__copyright__ = "2008, Jamie Bennett"
__author__ = "Jamie Bennett <jamie@linuxuk.org>"

import urllib
from xml.dom import minidom

from utils.theme import Theme

#The lookup url for the rss feed
WEATHER_LOOKUP_URL = "http://www.google.co.uk/ig/api?weather="

WEATHER_IMAGES = {'Mostly Sunny': 'weather-sun-mostly',
                  'Chance of Rain': 'weather-rain-chance',
                  'Partly Sunny': 'weather-sun-clouds',
                  'Rain': 'weather-rain',
                  'Chance of Storm': 'weather-lightning',
                  'Fog': 'weather-fog',
                  'Clear': 'weather-sun',
                  'Icy': 'weather-ice',
                  'Cloudy': 'weather-clouds',
                  'Snow': 'weather-snow'
				  }

class Weather:
    """
    Weather class
    @author: Jamie Bennett <jamie@linuxuk.org>
    """

    def __init__(self, location="", metric=""):
        """
        Sets up the weather object
        @param location (String) location of weather to get forecast
        for.
        """
        self.__location = ''
        self.__forecasts = []
        self.__metric = ''
        self.set_location(location)
        self.set_metric(metric)
        self.refresh()

    def find_forecast(self, search):
        """
        Returns the search results page for the search
        @param search String
        @return array of forecasts
        """
        url = WEATHER_LOOKUP_URL + search
        url = url.replace(" ", "%20")
        dom = minidom.parse(urllib.urlopen(url))

        in_forecast = 0
        day = ''
        low = ''
        high = ''
        condition = ''
        image = ''

        for node in dom.getElementsByTagNameNS('*', '*'):
            # a problem occured so bail out
            if (node.nodeName == 'problem_cause'):
                print 'Location not found or network problem'
                break

            if (node.nodeName == 'forecast_conditions'):
                in_forecast = 1

            if (in_forecast):
                if node.nodeName == 'day_of_week':
                    day = node.getAttribute('data')
                if node.nodeName == 'low':
                    low = node.getAttribute('data')
                    converted_low = self.weather_units (int(low))
                if node.nodeName == 'high':
                    high = node.getAttribute('data')
                    converted_high = self.weather_units (int(high))
                if node.nodeName == 'condition':
                    condition = node.getAttribute('data')
                    image = self.set_image (condition)

            # condition is the last element of the forecast we are
            # interested in so write out the forecast if this is set.
                if (condition):
                    forecast = {"Day":day,
                                "Low":str(converted_low),
                                "High":str(converted_high),
                                "Condition":condition,
                                "Image":image}
                    self.add_forecast(forecast)
                    in_forecast = 0
                    day = ''
                    low = ''
                    high = ''
                    condition = ''
                    image = ''

        return self.__forecasts

    def set_location(self, location):
        """
        Sets the current location
        @param type String should be a city, country name or
        post/zip code
        """
        self.__location = location

    def get_location(self):
        """
        Gets the current location
        @return type String of city, country name or post/zip code
        """
        return self.__location

    def set_metric(self, metric):
        """
        Sets the current metric flag
        @param type bool, true = use metric, false = imperial
        """
        self.__metric = metric

    def get_metric(self):
        """
        Gets the current metric flag
        @return type bool, true = use metric, false = imperial
        """
        return self.__metric

    def clear_forecasts(self):
        """
        Resets the forecast array
        """
        self.__forecasts = []

    def add_forecast(self, forecast):
        """
        Appends a forecast to the forecast array
        @param forecast dict
        """
        self.__forecasts.append(forecast)

    def get_forecasts(self):
        """
        Returns an array of forecasts
        @return array of forecasts
        """
        return self.__forecasts

    def weather_units (self, temp):
        """
        Convert (or not) a given imperial unit temp to metric depending
        on what the configuration file says to use for display units.
        @return type int converted (or not) weather temp
        """
        converted_temp = temp

        if (self.__metric):
            converted_temp = (temp-32) * (5.0/9)

        return int(round(converted_temp, 0))

    def set_image(self, condition):
        """
        Returns an image for a given weather condition.
        @return type String of weather image
        """
        try:
            image_name = WEATHER_IMAGES[condition]
        except KeyError:
            image_name = 'weather-na'

        image = Theme().getImage(image_name)

        return image

    def refresh(self):
        """
        Clear current weather and forecasts and then loads new data
        from xml or rss feed
        """
        if (self.__location):
            self.clear_forecasts()
            self.find_forecast(self.__location)
